﻿Namespace HHHB3.Filters
    Public Class all_filters
        ''' <summary>
        ''' Returns only Objects with a activ property and where activ = true
        ''' </summary>
        ''' <param name="obj_item"></param>
        ''' <returns></returns>
        Public Function ObjAktiv(ByVal obj_item As Object) As Boolean
            Dim type = obj_item.GetType
            Dim ref_eventinfo As Reflection.PropertyInfo = type.GetProperty("aktiv")
            If ref_eventinfo Is Nothing Then Return False
            Dim v_value As Boolean = CBool(ref_eventinfo.GetValue(obj_item, Nothing))
            Return v_value
        End Function

        ''' <summary>
        ''' Returns only Objects with a activ property and where activ = false
        ''' </summary>
        ''' <param name="obj_item"></param>
        ''' <returns></returns>
        Public Function ObjInaktiv(ByVal obj_item As Object) As Boolean
            Dim type = obj_item.GetType
            Dim ref_eventinfo As Reflection.PropertyInfo = type.GetProperty("aktiv")
            If ref_eventinfo Is Nothing Then Return False
            Dim v_value As Boolean = CBool(ref_eventinfo.GetValue(obj_item, Nothing))
            If v_value Then
                Return False
            Else
                Return True
            End If
        End Function

#Region "Decimals"
        Public Function TotAmountTimespanBiggerZero(ByVal obj_item As Object) As Boolean
            Dim type = obj_item.GetType
            Dim ref_eventinfo As Reflection.PropertyInfo = type.GetProperty("totamount_timespan")
            Dim ref_eventinfo_active As Reflection.PropertyInfo = type.GetProperty("aktiv")
            If ref_eventinfo Is Nothing OrElse ref_eventinfo_active Is Nothing Then Return False
            Dim v_value As Decimal = CDec(ref_eventinfo.GetValue(obj_item, Nothing))
            Dim v_aktiv As Boolean = CBool(ref_eventinfo_active.GetValue(obj_item, Nothing))
            If v_aktiv = True AndAlso v_value > 0 Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Function TotAmountTimespanSmallerZero(ByVal obj_item As Object) As Boolean
            Dim type = obj_item.GetType
            Dim ref_eventinfo As Reflection.PropertyInfo = type.GetProperty("totamount_timespan")
            Dim ref_eventinfo_active As Reflection.PropertyInfo = type.GetProperty("aktiv")
            If ref_eventinfo Is Nothing OrElse ref_eventinfo_active Is Nothing Then Return False
            Dim v_value As Decimal = CDec(ref_eventinfo.GetValue(obj_item, Nothing))
            Dim v_aktiv As Boolean = CBool(ref_eventinfo_active.GetValue(obj_item, Nothing))
            If v_aktiv = True AndAlso v_value < 0 Then
                Return True
            Else
                Return False
            End If
        End Function
#End Region

#Region "Kategorien"
        Public Function MainCatsAll(ByVal obj_item As Object) As Boolean
            Dim category As category_v01 = DirectCast(obj_item, category_v01)
            If category.parent_cat_id Is Nothing Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Function MainCatsAktiv(ByVal obj_item As Object) As Boolean
            Dim category As category_v01 = DirectCast(obj_item, category_v01)
            If category.parent_cat_id Is Nothing AndAlso category.aktiv Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Function CatsAktiv(ByVal obj_item As Object) As Boolean
            Dim category As category_v01 = DirectCast(obj_item, category_v01)
            If category.aktiv Then
                Return True
            Else
                Return False
            End If
        End Function
#End Region

#Region "Folder"

        Public Function FoldersAktiv(ByVal obj_item As Object) As Boolean
            Dim folder As folder_v01 = DirectCast(obj_item, folder_v01)
            If folder.aktiv Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Function FoldersInAktiv(ByVal obj_item As Object) As Boolean
            Dim folder As folder_v01 = DirectCast(obj_item, folder_v01)
            If folder.aktiv Then
                Return False
            Else
                Return True
            End If
        End Function
#End Region

#Region "Konto"
        Public Function KontoFavorit(ByVal obj_item As Object) As Boolean
            Dim konto As konto_v01 = DirectCast(obj_item, konto_v01)
            If konto.aktiv And konto.favorite Then
                Return True
            Else
                Return False
            End If
        End Function

        ''' <summary>
        ''' Checks if Konto is active and shall be displayed in overview
        ''' </summary>
        ''' <param name="obj_item">Konto to check</param>
        ''' <returns></returns>
        Public Function KontoOverview(ByVal obj_item As Object) As Boolean
            Dim konto As konto_v01 = DirectCast(obj_item, konto_v01)
            If konto.aktiv And konto.in_vermoegen Then
                Return True
            Else
                Return False
            End If
        End Function
#End Region

#Region "Transaction Repeat"

        ''' <summary>
        ''' Filters upcoming TransactionsRepeat with booking date within the next 30 days
        ''' </summary>
        ''' <param name="obj_item">Object must be of type transactions_repeat_v01</param>
        ''' <returns></returns>
        Public Function TransRepUpcoming(ByVal obj_item As Object) As Boolean
            Dim item As transactions_repeat_v01 = TryCast(obj_item, transactions_repeat_v01)
            If item IsNot Nothing Then
                Dim v_until As Date = Date.Now.AddMonths(1)
                If item.faelligkeit >= Date.Now And item.faelligkeit <= v_until Then
                    Return True
                Else
                    Return False
                End If
            Else
                Return False
            End If
        End Function

#End Region

#Region "Budgets"
        Public Function BudgetsAktiv(ByVal obj_item As Object) As Boolean
            Dim budget As budget_v01 = DirectCast(obj_item, budget_v01)
            If budget.aktiv Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Function BudgetsInAktiv(ByVal obj_item As Object) As Boolean
            Dim budget As budget_v01 = DirectCast(obj_item, budget_v01)
            If budget.aktiv Then
                Return False
            Else
                Return True
            End If
        End Function
#End Region

#Region "Messages"
        ''' <summary>
        ''' Returns only Messages with a unread state
        ''' </summary>
        ''' <param name="obj_item"></param>
        ''' <returns></returns>
        Public Function MessagesUnread(ByVal obj_item As Object) As Boolean
            Dim v_message As message_v01 = DirectCast(obj_item, message_v01)
            If v_message.isread Then
                Return False
            Else
                Return True
            End If
        End Function

        ''' <summary>
        ''' Returns only Objects which are read.
        ''' </summary>
        ''' <param name="obj_item"></param>
        ''' <returns></returns>
        Public Function MessagesRead(ByVal obj_item As Object) As Boolean
            Dim v_message As message_v01 = DirectCast(obj_item, message_v01)
            If v_message.isread Then
                Return True
            Else
                Return False
            End If
        End Function
#End Region

    End Class
End Namespace

