﻿Imports System.Reflection
Imports System.Collections.ObjectModel
Imports System.Runtime.CompilerServices
Imports System.Collections.Specialized
Imports System.Windows.Shell
Imports System.ComponentModel

''' <summary>
''' Klasse definiert ein Item vom Typ zuletzt benutzte Datenbank
''' </summary>
Public Class cls_lastdb

    Implements IEditableObject
    Implements INotifyPropertyChanged

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Private Sub NotifyPropertyChanged(<CallerMemberName()> Optional ByVal propertyName As String = Nothing)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub


    Dim v_pfad As String
    Public Property pfad As String
        Get
            Return v_pfad
        End Get
        Set(value As String)
            If v_pfad <> value Then
                v_pfad = value
                NotifyPropertyChanged("pfad")
            End If
        End Set
    End Property

    Dim v_dbname As String
    Public Property dbname As String
        Get
            Return v_dbname
        End Get
        Set(value As String)
            If v_dbname <> value Then
                v_dbname = value
                NotifyPropertyChanged("dbname")
            End If
        End Set
    End Property

    Dim v_pinned As Boolean
    Public Property pinned As Boolean
        Get
            Return v_pinned
        End Get
        Set(value As Boolean)
            If pinned <> value Then
                v_pinned = value
                NotifyPropertyChanged("pinned")
                NotifyPropertyChanged("pinned_str")
            End If
        End Set
    End Property

    Public ReadOnly Property pinned_str As String
        Get
            If v_pinned Then
                Return "Favoriten"
            Else
                Return "Zuletzt benutzte Datenbanken"
            End If
            'Return v_pinned.ToString
        End Get
    End Property

    Dim v_sorter As Integer
    Public Property sorter As Integer
        Get
            Return v_sorter
        End Get
        Set(value As Integer)
            If v_sorter <> value Then
                v_sorter = value
                NotifyPropertyChanged("sorter")
            End If
        End Set
    End Property

    Dim v_date As Date = Date.Now
    Public Property add_date As Date
        Get
            Return v_date
        End Get
        Set(value As Date)
            If v_date <> value Then
                v_date = value
                NotifyPropertyChanged("add_date")
            End If
        End Set
    End Property

    Sub New()

    End Sub

    Sub New(_pfad As String, _pinned As Boolean, Optional _sorter As Integer = 0)
        'dbname = _name
        pfad = _pfad
        pinned = _pinned
        sorter = _sorter
    End Sub

#Region "Edit Routinen"
    Private backup_props As Hashtable

    Public Sub BeginEdit() Implements IEditableObject.BeginEdit
        'enumerate properties
        Dim properties As PropertyInfo() = (Me.[GetType]()).GetProperties(BindingFlags.[Public] Or BindingFlags.Instance)

        backup_props = New Hashtable(properties.Length - 1)

        For i As Integer = 0 To properties.Length - 1
            'check if there is set accessor
            If properties(i).GetSetMethod() IsNot Nothing Then
                Dim value As Object = properties(i).GetValue(Me, Nothing)
                backup_props.Add(properties(i).Name, value)
            End If
        Next

    End Sub

    Public Sub EndEdit() Implements IEditableObject.EndEdit
        backup_props = Nothing
    End Sub

    Public Sub CancelEdit() Implements IEditableObject.CancelEdit
        'check for inappropriate call sequence
        If backup_props Is Nothing Then
            Return
        End If

        'restore old values
        Dim properties As PropertyInfo() = (Me.[GetType]()).GetProperties(BindingFlags.[Public] Or BindingFlags.Instance)
        For i As Integer = 0 To properties.Length - 1
            'check if there is set accessor
            If properties(i).GetSetMethod() IsNot Nothing Then
                Dim value As Object = backup_props(properties(i).Name)
                properties(i).SetValue(Me, value, Nothing)
            End If
        Next

        'delete current values
        backup_props = Nothing
    End Sub

#End Region

#Region "JumpPath"
    Public ReadOnly Property jump_path As JumpPath
        Get
            Dim v_temp As New JumpPath
            With v_temp
                .Path = pfad
                If pinned Then
                    .CustomCategory = "Angeheftet"
                Else
                    .CustomCategory = "Zuletzt benutzt"
                End If
            End With
            Return v_temp
        End Get
    End Property


#End Region


End Class
