﻿Imports System.IO
Imports System.Windows.Resources
Imports System.Windows.Shell

Module mod_help
    Public WithEvents myJumpList As New JumpList
    Public _Splash As splash
    Public _SplashVM As MainModel_Splash

    Public Sub CleanUpSplash()
        'On Error Resume Next
        _Splash.Close()
        'making objects inaccessible from code enables the GarbageCollector to clean them up
        _Splash = Nothing
        _SplashVM = Nothing
    End Sub

    'Global Variable to check if the the program is called by a former restart command:
    Public isRestart As Boolean = False

    ''' <summary>
    ''' Gets the value from a defined key from registry
    ''' Base Path is HKEY_CURRENT_USER\Software\MyAreaAt\HHHB3
    ''' </summary>
    ''' <param name="v_key">Name of the key to return</param>
    ''' <returns>Value of the key</returns>
    Public Function GetValueFromRegistryKey(v_key As String) As String
        If String.IsNullOrEmpty(v_key) Then Return Nothing
        Try
            Return My.Computer.Registry.GetValue("HKEY_CURRENT_USER\Software\MyAreaAt\HHHB3", v_key, "").ToString
        Catch ex As Exception
            cls_logger.v_logger.Error("GetValueFromRegistryKey failed with " & ex.Message)
            Return Nothing
        End Try
    End Function

    ''' <summary>
    ''' Sets the value of a defined key from registry
    ''' Base Path is HKEY_CURRENT_USER\Software\MyAreaAt\HHHB3
    ''' </summary>
    ''' <param name="v_key">Name of the key</param>
    ''' <param name="v_value">Value to set</param>
    ''' <returns>Value of the key</returns>
    Public Function SetValueFromRegistryKey(v_key As String, v_value As String) As Boolean
        If String.IsNullOrEmpty(v_key) Then Return Nothing
        Try
            My.Computer.Registry.SetValue("HKEY_CURRENT_USER\Software\MyAreaAt\HHHB3", v_key, v_value)
            Return True
        Catch ex As Exception
            cls_logger.v_logger.Error("SetValueFromRegistryKey failed with " & ex.Message)
            Return False
        End Try
    End Function


    ''' <summary>
    ''' Returns the Value of a given Resource (defined by Name) and returns a SolidColorBrush
    ''' if Resource is not found return Color.Black
    ''' </summary>
    ''' <param name="ResourceName">Name of the resource e.g. IdealForegroundColorBrush</param>
    ''' <returns>Colorbrush of the resource or black if result is not found</returns>
    Public Function ColorBrushFromResources(ResourceName As String) As SolidColorBrush
        Dim v_color As SolidColorBrush = TryCast(Application.Current.FindResource(ResourceName), SolidColorBrush)
        If v_color Is Nothing Then
            Return New SolidColorBrush(Colors.Black)
        Else
            Return v_color
        End If
    End Function



    ''' <summary> splashMessage will be omitted, if the splash isn't ready yet </summary>
    Public Sub TrySplashMessage(splashMessage As String)
        If _Splash Is Nothing Then Return
        If _SplashVM Is Nothing Then
            _SplashVM = DirectCast(_Splash.Dispatcher.Invoke(DirectCast(Function() _Splash.DataContext, Func(Of Object))), MainModel_Splash)
        End If
        cls_logger.v_logger.Info("Run: " & splashMessage)
        _SplashVM.Text = splashMessage
        _SplashVM.Prg_Now += 1
    End Sub

    ''' <summary>
    ''' Lädt den Windows Forms Datei auswahl Dialog zur Auswahl eines Bildes und gibt es als Byte() zurück
    ''' </summary>
    ''' <returns></returns>
    Public Function get_symbol_from_dialog() As Byte()
        Dim v_file_diag As New Microsoft.Win32.OpenFileDialog
        With v_file_diag
            .FileName = "symbol"
            .DefaultExt = ".png"
            .Filter = "Bilder|*.png;*.gif;*.jpg;*.jpeg;*.ico"
            .Multiselect = False
        End With

        Dim v_res As Boolean = CBool(v_file_diag.ShowDialog)
        If v_res = True Then
            Using filestream = New IO.FileStream(v_file_diag.FileName, IO.FileMode.Open)
                Dim buffer As Byte() = New Byte(CInt(filestream.Length - 1)) {}
                filestream.Read(buffer, 0, CInt(filestream.Length))
                Return buffer
            End Using
        Else
            Return Nothing
        End If
    End Function


    ''' <summary>
    ''' Converts an Image of Type byte() to BitmapImage
    ''' Can be used in FlowDocuments!
    ''' </summary>
    ''' <param name="imageData">The image stored as byte()</param>
    ''' <returns>BitmapImage</returns>
    Public Function ConvertImageBytetoBitmapImage(imageData As Byte()) As BitmapImage
        If imageData Is Nothing OrElse imageData.Length = 0 Then
            Return Nothing
        End If
        Dim image = New BitmapImage()
        Using mem = New MemoryStream(imageData)
            mem.Position = 0
            With image
                .BeginInit()
                .CreateOptions = BitmapCreateOptions.PreservePixelFormat
                .CacheOption = BitmapCacheOption.OnLoad
                .UriSource = Nothing
                .StreamSource = mem
                .EndInit()
            End With
        End Using
        image.Freeze()
        Return image
    End Function

    ''' <summary>
    ''' Konvertiert aus Resourcen ein Byte()
    ''' </summary>
    ''' <param name="v_filepath">z.B. /Resources/Icons8/Color/Speed_50px.png</param>
    ''' <returns></returns>
    Public Function Resource2Byte(v_filepath As String) As Byte()
        Dim _image_resource As StreamResourceInfo = Application.GetResourceStream(New Uri("HHHB3;component" & v_filepath, UriKind.Relative))
        Dim br As New BinaryReader(_image_resource.Stream)
        Dim bin As Byte() = br.ReadBytes(Convert.ToInt32(_image_resource.Stream.Length))
        _image_resource.Stream.Dispose()
        br.Dispose()
        Return bin
    End Function

    Public Function ByteToImage(imageData As Byte()) As ImageSource
        Dim biImg As New BitmapImage()
        Dim ms As New MemoryStream(imageData)
        biImg.BeginInit()
        biImg.StreamSource = ms
        biImg.EndInit()

        Dim imgSrc As ImageSource = TryCast(biImg, ImageSource)

        Return imgSrc
    End Function

    ''' <summary>
    ''' Converts a Resource from the image resourcedictionary to an byte() mainly used in symbol_ever Property of the different models
    ''' </summary>
    ''' <param name="v_item">Name (Key) of the Element</param>
    ''' <returns>Image converted to byte() or nothing if ResourceDictionary not found or key is wrong</returns>
    Public Function ResourceDictionaryImagetoByte(v_item As String) As Byte()
        If v_item Is Nothing OrElse v_item = Nothing OrElse v_item = String.Empty Then Return Nothing
        Dim imageset_current As New ResourceDictionary
        'Find Resourcedictionary of iconsets:
        For Each v_dict As ResourceDictionary In Application.Current.Resources.MergedDictionaries
            If v_dict.Source.ToString.Contains("Icons_") Then
                imageset_current = v_dict
                Exit For
            End If
        Next

        If imageset_current Is Nothing Then Return Nothing

        Dim data As Byte()
        Dim encoder As New PngBitmapEncoder()
        encoder.Frames.Add(BitmapFrame.Create(CType(imageset_current.Item(v_item), BitmapImage)))
        Using ms As New IO.MemoryStream()
            encoder.Save(ms)
            data = ms.ToArray()
        End Using
        Return data
    End Function

    ''' <summary>
    ''' Tries to find the given key in the Application.Current Resourcedictionary and returns it if type is BitmapImage
    ''' </summary>
    ''' <param name="v_key">Key to find</param>
    ''' <returns>Returns nothing if no image or not found</returns>
    Public Function FindResourceImage(v_key As String) As BitmapImage
        If v_key = String.Empty Then Return Nothing
        Dim v_image As BitmapImage = DirectCast(Application.Current.FindResource(v_key), BitmapImage)
        If v_image Is Nothing OrElse TypeOf (v_image) IsNot BitmapImage Then
            Return Nothing
        Else
            Return v_image
        End If

    End Function

    Public Function ConvertToDateTime(value As String) As Date
        Dim convertedDate As Date
        Try
            convertedDate = Convert.ToDateTime(value)
            Console.WriteLine("'{0}' converts to {1}.", value, convertedDate)
            Return convertedDate
        Catch e As FormatException
            Console.WriteLine("'{0}' is not in the proper format.", value)
            Return Nothing
        End Try
    End Function

End Module