﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema
Imports System.Text

''' <summary>
''' Entity defines an budget entry of a category within a budget.
''' It contains a scope a should, is comparison property
''' </summary>
Public Class budget2cat_2_value_v01
    Inherits model_base

    'Interne Variablen
    Dim v_id As Integer
    Dim v_b2c_id As Integer
    Dim v_scope_order As Integer = 1
    Dim v_should_amount As Decimal = 0D
    Dim v_bezeichnung As String
    Dim v_real_from As Date = Nothing
    Dim v_real_to As Date = Nothing


#Region "Mapped"
    <Key>
    Public Property Id As Integer
        Get
            Return v_id
        End Get
        Set(value As Integer)
            If v_id <> value Then
                v_id = value
                NotifyPropertyChanged("Id")
            End If
        End Set
    End Property

    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If value <> v_bezeichnung Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
            End If
        End Set
    End Property

    Public Property b2c_id As Integer
        Get
            Return v_b2c_id
        End Get
        Set(value As Integer)
            If v_b2c_id <> value Then
                v_b2c_id = value
                NotifyPropertyChanged("b2c_id")
            End If
        End Set
    End Property

    Public Property scope_order As Integer
        Get
            Return v_scope_order
        End Get
        Set(value As Integer)
            If v_scope_order <> value Then
                v_scope_order = value
                NotifyPropertyChanged("scope_order")
            End If
        End Set
    End Property

    Public Property should_amount As Decimal
        Get
            Return v_should_amount
        End Get
        Set(value As Decimal)
            If v_should_amount <> value Then
                v_should_amount = value
                NotifyPropertyChanged("should_amount")
                NotifyPropertyChanged("delta_amount")
            End If
        End Set
    End Property

    Public Property real_from As Date
        Get
            Return v_real_from
        End Get
        Set(value As Date)
            If v_real_from <> value Then
                v_real_from = value
                NotifyPropertyChanged("real_from")
                NotifyPropertyChanged("should_amount")
                NotifyPropertyChanged("is_amount")
                NotifyPropertyChanged("delta_amount")
            End If
        End Set
    End Property

    Public Property real_to As Date
        Get
            Return v_real_to
        End Get
        Set(value As Date)
            If v_real_to <> value Then
                v_real_to = value
                NotifyPropertyChanged("real_to")
                NotifyPropertyChanged("should_amount")
                NotifyPropertyChanged("is_amount")
                NotifyPropertyChanged("delta_amount")
            End If
        End Set
    End Property


    Public ReadOnly Property is_amount As Decimal
        Get
            'Returns the current amount of for the current timeframe
            If real_from = Nothing OrElse real_to = Nothing Then Return 0D
            If nv_budget_2_cat_back.nv_category_2_budcat_back Is Nothing Then Return 0D
            Dim v_cat As category_v01 = nv_budget_2_cat_back.nv_category_2_budcat_back
            Dim v_amount As Decimal = 0D
            'Check if the category should be summarized:
            v_amount = v_cat.nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.datum >= real_from AndAlso x.nv_trans_t2k_back.datum <= real_to).Sum(Function(y) y.amount)
            If nv_budget_2_cat_back.sumchilds AndAlso v_cat.nv_subcats IsNot Nothing AndAlso v_cat.nv_subcats.Count > 0 Then
                For Each v_subcat As category_v01 In v_cat.nv_subcats
                    v_amount += v_subcat.nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.datum >= real_from AndAlso x.nv_trans_t2k_back.datum <= real_to).Sum(Function(y) y.amount)
                Next
            End If
            Return v_amount
        End Get
    End Property

    Public ReadOnly Property delta_amount As Decimal
        Get
            Dim v_isamount As Decimal = is_amount
            'If v_isamount = Nothing Then Return 0
            'If should_amount = Nothing Then Return 0
            If nv_budget_2_cat_back.income Then
                Return v_isamount - should_amount
            Else
                Return should_amount - v_isamount
            End If
        End Get
    End Property

    Public ReadOnly Property tooltiptext As String
        Get
            Dim v_str As New StringBuilder(bezeichnung)
            v_str.AppendLine("")
            v_str.AppendLine(String.Format("Zeitraum von: {0}", real_from.ToShortDateString))
            v_str.AppendLine(String.Format("Zeitraum bis: {0}", real_to.ToShortDateString))
            Return v_str.ToString
        End Get
    End Property
#End Region

    Public Sub notify_all()
        NotifyPropertyChanged("should_amount")
        NotifyPropertyChanged("is_amount")
        NotifyPropertyChanged("delta_amount")
        NotifyPropertyChanged("tooltiptext")
    End Sub

#Region "Navigation Properties"
    Dim v_nv_budget_2_cat_back As budget_2_cat_v01

    <ForeignKey("b2c_id")>
    Public Property nv_budget_2_cat_back As budget_2_cat_v01
        Get
            Return v_nv_budget_2_cat_back
        End Get
        Set(value As budget_2_cat_v01)
            If v_nv_budget_2_cat_back IsNot value Then
                v_nv_budget_2_cat_back = value
                NotifyPropertyChanged("nv_budget_2_cat_back")
            End If
        End Set
    End Property
#End Region

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "should_amount"
                    If should_amount = Nothing Then result = "Soll-Betrag darf nicht leer sein"
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

End Class
