﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema
Imports System.IO
Imports System.Xml
Imports System.Text
Imports System.Windows.Markup
Imports LiveCharts
Imports System.Collections.Specialized

Public Class budget_v01
    Inherits model_base

    'Interne Variablen
    Dim v_bud_id As Integer
    Dim v_bezeichnung As String
    Dim v_color As String = "#000000"
    Dim v_symbol As Byte()
    Dim v_start_date As Date = New Date(Date.Now.Year, 1, 1)
    Dim v_end_date As Date = New Date(Date.Now.Year, 12, 31)
    Dim v_comment As String
    Dim v_aktiv As Nullable(Of Boolean) = True
    Dim v_budget_scope As String

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "bezeichnung"
                    If String.IsNullOrEmpty(bezeichnung) Then result = "Bezeichnung darf nicht leer sein"
                Case "start_date"
                    If start_date = Nothing Then result = "Startdatum darf nicht leer sein"
                Case "end_date"
                    If end_date = Nothing Then result = "Enddatum darf nicht leer sein"
            End Select

            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property bud_id As Integer
        Get
            Return v_bud_id
        End Get
        Set(value As Integer)
            If v_bud_id <> value Then
                v_bud_id = value
                NotifyPropertyChanged("bud_id")
            End If
        End Set
    End Property

    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If v_bezeichnung <> value Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
            End If
        End Set
    End Property

    Public Property color As String
        Get
            Return v_color
        End Get
        Set(value As String)
            If v_color <> value Then
                v_color = value
                NotifyPropertyChanged("color")
            End If
        End Set
    End Property

    <MaxLength>
    Public Property symbol As Byte()
        Get
            Return v_symbol
        End Get
        Set(value As Byte())
            If v_symbol IsNot value Then
                v_symbol = value
                NotifyPropertyChanged("symbol")
                NotifyPropertyChanged("symbol_ever")
            End If
        End Set
    End Property

    Public Property start_date As Date
        Get
            Return v_start_date
        End Get
        Set(value As Date)
            If v_start_date <> value Then
                v_start_date = value
                NotifyPropertyChanged("start_date")
                NotifyPropertyChanged("scope_based_value_amount")
            End If
        End Set
    End Property

    Public Property end_date As Date
        Get
            Return v_end_date
        End Get
        Set(value As Date)
            If v_end_date <> value Then
                v_end_date = value
                NotifyPropertyChanged("end_date")
                NotifyPropertyChanged("scope_based_value_amount")
            End If
        End Set
    End Property

    Public Property comment As String
        Get
            Return v_comment
        End Get
        Set(value As String)
            If v_comment <> value Then
                v_comment = value
                NotifyPropertyChanged("comment")
            End If
        End Set
    End Property

    Public Property aktiv As Nullable(Of Boolean)
        Get
            Return v_aktiv
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_aktiv.Equals(value) Then
                v_aktiv = value
                NotifyPropertyChanged("aktiv")
            End If
        End Set
    End Property

    Public Property budget_scope As String
        Get
            Return v_budget_scope
        End Get
        Set(value As String)
            If v_budget_scope <> value Then
                v_budget_scope = value
                NotifyPropertyChanged("budget_scope")
                NotifyPropertyChanged("scope_based_value_amount")
            End If
        End Set
    End Property
#End Region

#Region "Navigation Properties"
    Public Overridable Property nv_budget_2_budcat As New ObservableCollection(Of budget_2_cat_v01)

    <NotMapped>
    Public Property BudgetCategories As ListCollectionView

    '<NotMapped>
    'Public Property BudgetIncomeCategories As ListCollectionView
    '<NotMapped>
    'Public Property BudgetExpendCategories As ListCollectionView

    Private Function FilterIncomeCats(ByVal obj_item As Object) As Boolean
        Return DirectCast(obj_item, budget_2_cat_v01).income
    End Function

    Private Function FilterExpendCats(ByVal obj_item As Object) As Boolean
        If DirectCast(obj_item, budget_2_cat_v01).income Then
            Return False
        Else
            Return True
        End If
    End Function


#End Region

#Region "Extension"

    ''' <summary>
    ''' If the budgets has categories the value is false
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property changes_allowed As Boolean
        Get
            If nv_budget_2_budcat.Count > 0 Then
                Return False
            Else
                Return True
            End If
        End Get
    End Property

    ''' <summary>
    ''' Calculates the amount of periodes based on start_and end_date 
    ''' e.g. Start Date is 1.1.2018 and End Date is 31.12.2018 with scope month
    ''' would give back 12
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property scope_based_value_amount As Long
        Get
            If budget_scope = String.Empty OrElse start_date = Nothing OrElse end_date = Nothing OrElse end_date < start_date Then Return Nothing

            Select Case budget_scope
                Case "week"
                    Return DateDiff(DateInterval.WeekOfYear, start_date, end_date)
                Case "month"
                    Return DateDiff(DateInterval.Month, start_date, end_date) + 1
                Case "quarteryear"
                    Dim v_yearcount As Long = DateDiff(DateInterval.Year, start_date, end_date)
                    If v_yearcount = 0 Then
                        v_yearcount = 1
                    Else
                        v_yearcount += 1
                    End If
                    Return v_yearcount * 4
                Case "halfyear"
                    Dim v_yearcount As Long = DateDiff(DateInterval.Year, start_date, end_date)
                    If v_yearcount = 0 Then
                        v_yearcount = 1
                    Else
                        v_yearcount += 1
                    End If
                    Return v_yearcount * 2
                Case "year"
                    Dim v_yearcount As Long = DateDiff(DateInterval.Year, start_date, end_date)
                    If v_yearcount = 0 Then
                        v_yearcount = 1
                    Else
                        v_yearcount += 1
                    End If
                    Return v_yearcount
                Case Else
                    Return Nothing
            End Select
        End Get
    End Property

    Dim v_tooltiptext_budget As String
    Public ReadOnly Property tooltip_budget As String
        Get
            'Tooltip zusammenbauen:
            v_tooltiptext_budget = "Startzeit: " & start_date.ToString("dd.MM.yyyy") & Environment.NewLine & "Endzeit: " & end_date.ToString("dd.MM.yyyy")
            Return v_tooltiptext_budget
        End Get
    End Property

    Public ReadOnly Property symbol_ever() As Byte()
        Get
            If symbol Is Nothing Then
                Return ResourceDictionaryImagetoByte("Budget")
            Else
                Return symbol
            End If
        End Get
    End Property

    Sub New()
        CanClose = True
        IsClosed = False
        nv_budget_2_budcat_sorted = New ListCollectionView(nv_budget_2_budcat)
        _Collections = New INotifyCollectionChanged() {nv_budget_2_budcat}
        With nv_budget_2_budcat_sorted
            '            .GroupDescriptions.Add(New PropertyGroupDescription("nv_category_2_budcat_back.budget_maincat_name"))
            .GroupDescriptions.Add(New PropertyGroupDescription("maincatname"))
            '.LiveGroupingProperties.Add("maincatname")
            '.IsLiveGrouping = True
        End With

        'Changed in Version V1.0.0.25 because haptic was bas
        'BudgetIncomeCategories = New ListCollectionView(nv_budget_2_budcat) With {.Filter = New Predicate(Of Object)(AddressOf FilterIncomeCats)}
        'BudgetExpendCategories = New ListCollectionView(nv_budget_2_budcat) With {.Filter = New Predicate(Of Object)(AddressOf FilterExpendCats)}
        BudgetCategories = New ListCollectionView(nv_budget_2_budcat)
        BudgetCategories.SortDescriptions.Add(New ComponentModel.SortDescription("income", ComponentModel.ListSortDirection.Descending))
        BudgetCategories.SortDescriptions.Add(New ComponentModel.SortDescription("nv_category_2_budcat_back.fullname", ComponentModel.ListSortDirection.Descending))
    End Sub

    <NotMapped>
    Public Property nv_budget_2_budcat_sorted As ListCollectionView


    Public ReadOnly Property solid_color() As SolidColorBrush
        Get
            If color Is Nothing Then
                Return ColorBrushFromResources("IdealForegroundColorBrush")
            Else
                Try
                    Dim v_col As System.Drawing.Color = System.Drawing.ColorTranslator.FromHtml(color)
                    Dim v_color As Color = System.Windows.Media.Color.FromArgb(v_col.A, v_col.R, v_col.G, v_col.B)
                    Return New SolidColorBrush(v_color)
                Catch ex As Exception
                    Return ColorBrushFromResources("IdealForegroundColorBrush")
                End Try
            End If
        End Get
    End Property

    'Property für den Busy Indicator:
    Dim v_busy As Boolean = False
    <NotMapped>
    Property is_busy As Boolean
        Get
            Return v_busy
        End Get
        Set(value As Boolean)
            If v_busy <> value Then
                v_busy = value
                NotifyPropertyChanged("is_busy")
            End If
        End Set
    End Property
#End Region

#Region "Menüsystem"
    <NotMapped>
    Public Property cmdClose() As New RelayCommand(AddressOf Close, Function() CanClose = True)

    Private _isclosed As Boolean

    <NotMapped>
    Public Property IsClosed() As Boolean
        Get
            Return _isclosed
        End Get
        Set(value As Boolean)
            If _isclosed <> value Then
                _isclosed = value
                NotifyPropertyChanged("IsClosed")
            End If
        End Set
    End Property

    Private _CanClose As Boolean = True

    <NotMapped>
    Public Property CanClose() As Boolean
        Get
            Return _CanClose
        End Get
        Set
            If _CanClose <> Value Then
                _CanClose = Value
                NotifyPropertyChanged("CanClose")
            End If
        End Set
    End Property

    Public Sub Close()
        Me.IsClosed = True
        budget_report = Nothing

        NotifyPropertyChanged("IsClosed")
    End Sub
#End Region

    ''' <summary>
    ''' Property for Archiv to define the String Type of the Object
    ''' </summary>
    ''' <returns></returns>
    <NotMapped>
    Public ReadOnly Property object_type As String
        Get
            Return "Budget"
        End Get
    End Property


    'Property which holds the flowdocument of the report:
    Dim v_budget_report As FlowDocument
    <NotMapped>
    Public Property budget_report As FlowDocument
        Get
            Return v_budget_report
        End Get
        Set(value As FlowDocument)
            v_budget_report = value
            NotifyPropertyChanged("budget_report")
        End Set
    End Property

    Public ReadOnly Property budget_report_clone As FlowDocument
        Get
            If budget_report Is Nothing Then Return Nothing
            Dim str As String = XamlWriter.Save(budget_report)

            ' Load 
            Dim stringReader As New StringReader(str)
            Dim xmlReader As XmlReader = XmlReader.Create(stringReader)
            Dim cloneDocument As FlowDocument = CType(XamlReader.Load(xmlReader), FlowDocument)
            Return cloneDocument
        End Get
    End Property

    Public Sub notify_changes()
        NotifyPropertyChanged("budget_report_clone")
        NotifyPropertyChanged("Chart_Series_in")
        NotifyPropertyChanged("Chart_Labels_in")
        NotifyPropertyChanged("Chart_Formatter_in")
        NotifyPropertyChanged("Chart_Series_out")
        NotifyPropertyChanged("Chart_Labels_out")
        NotifyPropertyChanged("Chart_Formatter_out")
        'Budget Values:
        NotifyPropertyChanged("sum_should")
        NotifyPropertyChanged("sum_is")
        NotifyPropertyChanged("sum_should_income")
        NotifyPropertyChanged("sum_is_income")
        NotifyPropertyChanged("sum_delta_income")
        NotifyPropertyChanged("sum_should_expend")
        NotifyPropertyChanged("sum_is_expend")
        NotifyPropertyChanged("sum_delta_expend")
    End Sub


    Public Shared Sub AddDocument(ByVal from As FlowDocument, ByVal [to] As FlowDocument)
        Dim range As TextRange = New TextRange(from.ContentStart, from.ContentEnd)
        Dim stream As MemoryStream = New MemoryStream()
        System.Windows.Markup.XamlWriter.Save(range, stream)
        range.Save(stream, DataFormats.XamlPackage)
        Dim range2 As TextRange = New TextRange([to].ContentEnd, [to].ContentEnd)
        range2.Load(stream, DataFormats.XamlPackage)
    End Sub

    Public Shared Sub AddBlock(ByVal from As Block, ByVal [to] As FlowDocument)
        If from IsNot Nothing Then
            Dim range As TextRange = New TextRange(from.ContentStart, from.ContentEnd)
            Dim stream As MemoryStream = New MemoryStream()
            System.Windows.Markup.XamlWriter.Save(range, stream)
            range.Save(stream, DataFormats.XamlPackage)
            Dim textRange2 As TextRange = New TextRange([to].ContentEnd, [to].ContentEnd)
            textRange2.Load(stream, DataFormats.XamlPackage)
        End If
    End Sub

    Private Sub budget_v01_CollectionChanged(sender As Object, e As NotifyCollectionChangedEventArgs) Handles Me.CollectionChanged
        'BudgetIncomeCategories.Filter = New Predicate(Of Object)(AddressOf FilterIncomeCats)
        'BudgetExpendCategories.Filter = New Predicate(Of Object)(AddressOf FilterExpendCats)
        If Not BudgetCategories Is Nothing Then
            If Not BudgetCategories.MoveCurrentToNext Then BudgetCategories.MoveCurrentToPrevious()
            BudgetCategories.SortDescriptions.Clear()
            BudgetCategories.SortDescriptions.Add(New ComponentModel.SortDescription("income", ComponentModel.ListSortDirection.Descending))
            BudgetCategories.SortDescriptions.Add(New ComponentModel.SortDescription("nv_category_2_budcat_back.fullname", ComponentModel.ListSortDirection.Descending))
        End If
    End Sub

    Public ReadOnly Property sum_should As Decimal
        Get
            Return nv_budget_2_budcat.Sum(Function(x) x.nv_budget2cat_2_value.Sum(Function(z) z.should_amount))
        End Get
    End Property

    Public ReadOnly Property sum_is As Decimal
        Get
            Return nv_budget_2_budcat.Sum(Function(x) x.nv_budget2cat_2_value.Sum(Function(z) z.is_amount))
        End Get
    End Property

    Public ReadOnly Property sum_delta As Decimal
        Get
            Return sum_should - sum_is
        End Get
    End Property

    Public ReadOnly Property sum_should_income As Decimal
        Get
            Return nv_budget_2_budcat.Where(Function(x) x.income).Sum(Function(y) y.nv_budget2cat_2_value.Sum(Function(z) z.should_amount))
        End Get
    End Property

    Public ReadOnly Property sum_is_income As Decimal
        Get
            Return nv_budget_2_budcat.Where(Function(x) x.income).Sum(Function(y) y.nv_budget2cat_2_value.Sum(Function(z) z.is_amount))
        End Get
    End Property

    Public ReadOnly Property sum_delta_income As Decimal
        Get
            Return sum_should_income - sum_is_income
        End Get
    End Property

    Public ReadOnly Property sum_should_expend As Decimal
        Get
            Return nv_budget_2_budcat.Where(Function(x) x.income = False).Sum(Function(y) y.nv_budget2cat_2_value.Sum(Function(z) z.should_amount))
        End Get
    End Property

    Public ReadOnly Property sum_is_expend As Decimal
        Get
            Return nv_budget_2_budcat.Where(Function(x) x.income = False).Sum(Function(y) y.nv_budget2cat_2_value.Sum(Function(z) z.is_amount))
        End Get
    End Property

    Public ReadOnly Property sum_delta_expend As Decimal
        Get
            Return sum_should_expend - sum_is_expend
        End Get
    End Property


#Region "Diagram"
    'Region Contains all necessary diagramm Properties:
    'Properties of the Widget himself:

    'Income Chart:
    Public Property Chart_Series_in As SeriesCollection = New SeriesCollection
    Public Property Chart_Labels_in As New List(Of String)
    Public Property Chart_Formatter_in As Func(Of Double, String) = Function(value) value.ToString("N")

    'Expenditure Charts:
    Public Property Chart_Series_out As SeriesCollection = New SeriesCollection
    Public Property Chart_Labels_out As New List(Of String)
    Public Property Chart_Formatter_out As Func(Of Double, String) = Function(value) value.ToString("N")
#End Region

End Class
