﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema

Public Class person_v01
    Inherits model_base

    'Interne Variablen
    Dim v_pers_id As Integer
    Dim v_bezeichnung As String
    Dim v_color As String = "#000000"
    Dim v_symbol As Byte()
    Dim v_aktiv As Nullable(Of Boolean) = True
    Dim v_predefined As Nullable(Of Boolean) = False

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "name"
                    If String.IsNullOrEmpty(bezeichnung) Then result = "Name darf nicht leer sein"
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property pers_id As Integer
        Get
            Return v_pers_id
        End Get
        Set(value As Integer)
            If v_pers_id <> value Then
                v_pers_id = value
                NotifyPropertyChanged("pers_id")
            End If
        End Set
    End Property

    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If v_bezeichnung <> value Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
                NotifyPropertyChanged("bezeichnung_gal")
            End If
        End Set
    End Property

    Public Property color As String
        Get
            Return v_color
        End Get
        Set(value As String)
            If v_color <> value Then
                v_color = value
                NotifyPropertyChanged("color")
                NotifyPropertyChanged("solid_color")
            End If
        End Set
    End Property

    <MaxLength>
    Public Property symbol As Byte()
        Get
            Return v_symbol
        End Get
        Set(value As Byte())
            If v_symbol IsNot value Then
                v_symbol = value
                NotifyPropertyChanged("symbol")
                NotifyPropertyChanged("symbol_ever")
            End If
        End Set
    End Property

    Public Property aktiv As Nullable(Of Boolean)
        Get
            Return v_aktiv
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_aktiv.Equals(value) Then
                v_aktiv = value
                NotifyPropertyChanged("aktiv")
            End If
        End Set
    End Property

    Public Property predefined As Nullable(Of Boolean)
        Get
            Return v_predefined
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_predefined.Equals(value) Then
                v_predefined = value
                NotifyPropertyChanged("predefined")
            End If
        End Set
    End Property
#End Region

#Region "Navigation Properties"
    Public Overridable Property nv_person_trans As New ObservableCollection(Of transactions_v01)
    Public Overridable Property trans_rep As New ObservableCollection(Of transactions_repeat_v01)
    Public Overridable Property nv_person_2_transtemp As New ObservableCollection(Of transactions_template_v01)
#End Region

#Region "Extension"
    ''' <summary>
    ''' Property offers a SolidColorBrush out of the String HTML Color to change Foreground Color of an Item.
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property solid_color() As SolidColorBrush
        Get
            If color Is Nothing Then
                Return ColorBrushFromResources("IdealForegroundColorBrush")
            Else
                Try
                    Dim v_col As System.Drawing.Color = System.Drawing.ColorTranslator.FromHtml(color)
                    Dim v_color As Color = System.Windows.Media.Color.FromArgb(v_col.A, v_col.R, v_col.G, v_col.B)
                    Return New SolidColorBrush(v_color)
                Catch ex As Exception
                    Return ColorBrushFromResources("IdealForegroundColorBrush")
                End Try
            End If
        End Get
    End Property

    Public ReadOnly Property symbol_ever() As Byte()
        Get
            If symbol Is Nothing Then
                Return ResourceDictionaryImagetoByte("User")
            Else
                Return symbol
            End If
        End Get
    End Property

    ''' <summary>
    ''' Returns the totamount of a person depending on a specified timespan
    ''' also sets the totamount_timespan property to the result.
    ''' e.g. if you calcutlate a graph which is not bound directly to the model to can use the totamount_timespan property as a temporary storage for the totamount to enable sorting.
    ''' </summary>
    ''' <param name="v_begin">Begindate must be smaller or equal to end date</param>
    ''' <param name="v_end">Enddate muust be greater or equal to begin date</param>
    ''' <param name="v_selected_multiplier">Set 1 if income, set -1 if expend, set 0 if transfer</param>
    ''' <returns>Decimal Value of Totalamount of a category</returns>
    Public Function totamount_timespan_caluclator(v_begin As Date, v_end As Date, v_selected_multiplier As Short) As Decimal
        If v_begin = Nothing Then Return 0
        If v_end = Nothing Then Return 0

        If nv_person_trans Is Nothing OrElse nv_person_trans.Count = 0 Then
            Return 0
        Else
            Dim v_value As Decimal = nv_person_trans.Where(Function(x) x.datum >= v_begin AndAlso x.datum <= v_end AndAlso x.nv_transtyp_trans_back.multiplier = v_selected_multiplier).Sum(Function(x) x.real_totamount)
            v_totamount_timespan = v_value
            NotifyPropertyChanged("totamount_timespan")
            Return v_value
        End If
    End Function

    Dim v_totamount_timespan As Decimal = 0
    Public ReadOnly Property totamount_timespan As Decimal
        Get
            Return v_totamount_timespan
        End Get
    End Property

    Sub New()

    End Sub

    'Konstruktor für die person_v01_balance Klasse
    Sub New(v_person As person_v01)
        With v_person
            aktiv = .aktiv
            color = .color
            bezeichnung = .bezeichnung
            nv_person_2_transtemp = .nv_person_2_transtemp
            nv_person_trans = .nv_person_trans
            pers_id = .pers_id
            symbol = .symbol
            trans_rep = .trans_rep
        End With
    End Sub

    ''' <summary>
    ''' bezeichnung Property suitable for use in ribbon
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property bezeichnung_gal As String
        Get
            If bezeichnung.Length > 10 Then
                Return bezeichnung.Substring(0, 10)
            Else
                Return bezeichnung
            End If
        End Get
    End Property
#End Region
End Class
