﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema

Public Class receiver_v01
    Inherits model_base

    'Interne Variablen
    Dim v_rec_id As Integer
    Dim v_bezeichnung As String
    Dim v_aktiv As Nullable(Of Boolean) = True
    Dim v_std_kat_id As Nullable(Of Integer)
    Dim v_predefined As Nullable(Of Boolean) = False

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "bezeichnung"
                    If String.IsNullOrEmpty(bezeichnung) Then result = "Bezeichnung darf nicht leer sein"
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property rec_id As Integer
        Get
            Return v_rec_id
        End Get
        Set(value As Integer)
            If v_rec_id <> value Then
                v_rec_id = value
                NotifyPropertyChanged("rec_id")
            End If
        End Set
    End Property

    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If v_bezeichnung <> value Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
            End If
        End Set
    End Property

    Public Property aktiv As Nullable(Of Boolean)
        Get
            Return v_aktiv
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_aktiv.Equals(value) Then
                v_aktiv = value
                NotifyPropertyChanged("aktiv")
            End If
        End Set
    End Property

    Public Property std_kat_id As Nullable(Of Integer)
        Get
            Return v_std_kat_id
        End Get
        Set(value As Nullable(Of Integer))
            If v_std_kat_id <> value Then
                v_std_kat_id = value
                NotifyPropertyChanged("std_kat_id")
            End If
        End Set
    End Property

    Public Property predefined As Nullable(Of Boolean)
        Get
            Return v_predefined
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_predefined.Equals(value) Then
                v_predefined = value
                NotifyPropertyChanged("predefined")
            End If
        End Set
    End Property
#End Region

#Region "Not Mapped"
    ''' <summary>
    ''' Returns the totamount of a receiver depending on a specified timespan
    ''' also sets the totamount_timespan property to the result.
    ''' e.g. if you calcutlate a graph which is not bound directly to the model to can use the totamount_timespan property as a temporary storage for the totamount to enable sorting.
    ''' </summary>
    ''' <param name="v_begin">Begindate must be smaller or equal to end date</param>
    ''' <param name="v_end">Enddate muust be greater or equal to begin date</param>
    ''' <returns>Decimal Value of Totalamount of a category</returns>
    Public Function totamount_timespan_caluclator(v_begin As Date, v_end As Date) As Decimal
        If v_begin = Nothing Then Return 0
        If v_end = Nothing Then Return 0

        If nv_receiver_trans Is Nothing OrElse nv_receiver_trans.Count = 0 Then
            Return 0
        Else
            Dim v_value As Decimal = nv_receiver_trans.Where(Function(x) x.datum >= v_begin AndAlso x.datum <= v_end).Sum(Function(x) x.real_totamount)
            v_totamount_timespan = v_value
            NotifyPropertyChanged("totamount_timespan")
            Return v_value
        End If
    End Function

    Dim v_totamount_timespan As Decimal = 0
    Public ReadOnly Property totamount_timespan As Decimal
        Get
            Return v_totamount_timespan
        End Get
    End Property


#End Region

#Region "Navigation Properties"
    Public Overridable Property nv_receiver_trans As New ObservableCollection(Of transactions_v01)
    Public Overridable Property trans_reps As New ObservableCollection(Of transactions_repeat_v01)
    Public Overridable Property nv_receiver_2_transtemp As New ObservableCollection(Of transactions_template_v01)
#End Region
End Class
