﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema

Public Class savingtargets_v01
    Inherits model_base

    'Interne Variablen
    Dim v_id As Integer
    Dim v_kto_id As Integer
    Dim v_comment As String
    Dim v_creation_date As Date = Date.Now
    Dim v_bezeichnung As String

    Dim v_scope As Integer
    Dim v_scope_free_start As Date = Date.Now
    Dim v_scope_free_end As Date = Date.Now.AddDays(1)

    Dim v_track_income As Boolean = True

    Dim v_target_value As Decimal = 0D
    Dim v_notify_progress As Boolean = True


#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "kto_id"
                    If kto_id = Nothing Then result = "Konto muss ausgewählt sein"
                Case "scope"
                    If scope = Nothing Then result = "Zeitraum muss ausgewählt sein"
                Case "target_value"
                    If target_value = Nothing Then result = "Zielbetrag muss angegeben sein"
                Case "bezeichnung"
                    If String.IsNullOrEmpty(bezeichnung) Then result = "Bezeichnung darf nicht leer sein"
                Case Else
                    Return result
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If

            NotifyPropertyChanged("hasErrors")
            Debug.Print("KontoErrorItem hasErrors " & v_haserrors.ToString)
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property Id As Integer
        Get
            Return v_id
        End Get
        Set(value As Integer)
            If v_id <> value Then
                v_id = value
                NotifyPropertyChanged("Id")
            End If
        End Set
    End Property


    Public Property kto_id As Integer
        Get
            Return v_kto_id
        End Get
        Set(value As Integer)
            v_kto_id = value
            NotifyPropertyChanged("kto_id")
        End Set
    End Property

    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If v_bezeichnung <> value Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
            End If
        End Set
    End Property

    Public Property comment As String
        Get
            Return v_comment
        End Get
        Set(value As String)
            If v_comment <> value Then
                v_comment = value
                NotifyPropertyChanged("comment")
                NotifyPropertyChanged("short_comment")
            End If
        End Set
    End Property

    Public Property creation_datum As Date
        Get
            Return v_creation_date
        End Get
        Set(value As Date)
            If v_creation_date <> value Then
                v_creation_date = value
                NotifyPropertyChanged("creation_datum")
            End If
        End Set
    End Property

    Public Property scope As Integer
        Get
            Return v_scope
        End Get
        Set(value As Integer)
            If v_scope <> value Then
                v_scope = value
                NotifyPropertyChanged("scope")
            End If
        End Set
    End Property

    Public Property scope_free_start As Date
        Get
            Return v_scope_free_start
        End Get
        Set(value As Date)
            If v_scope_free_start <> value Then
                v_scope_free_start = value
                NotifyPropertyChanged("scope_free_start")
            End If
        End Set
    End Property

    Public Property scope_free_end As Date
        Get
            Return v_scope_free_end
        End Get
        Set(value As Date)
            If v_scope_free_end <> value Then
                v_scope_free_end = value
                NotifyPropertyChanged("scope_free_end")
            End If
        End Set
    End Property

    Public Property track_income As Boolean
        Get
            Return v_track_income
        End Get
        Set(value As Boolean)
            If v_track_income <> value Then
                v_track_income = value
                NotifyPropertyChanged("track_income")
                NotifyPropertyChanged("target_value")
            End If
        End Set
    End Property

    Public Property target_value As Decimal
        Get
            Return v_target_value
        End Get
        Set(value As Decimal)
            'Depending if the target should track income or expends check if value is negative or positive:
            v_target_value = value
            If track_income Then
                If value < 0 Then
                    v_target_value = value * -1
                End If
            Else
                If value > 0 Then
                    v_target_value = value * -1
                End If
            End If
            NotifyPropertyChanged("target_value")
            NotifyPropertyChanged("current_value_perc")
        End Set
    End Property

    Public Property notify_progress As Boolean
        Get
            Return v_notify_progress
        End Get
        Set(value As Boolean)
            If v_notify_progress <> value Then
                v_notify_progress = value
                NotifyPropertyChanged("notify_progress")
            End If
        End Set
    End Property

#End Region


#Region "Navigation Properties"

    Dim v_nv_target_2_konto As konto_v01

    <ForeignKey("kto_id")>
    Public Property nv_target_2_konto As konto_v01
        Get
            Return v_nv_target_2_konto
        End Get
        Set(value As konto_v01)
            If v_nv_target_2_konto IsNot value Then
                v_nv_target_2_konto = value
                NotifyPropertyChanged("nv_target_2_konto")
            End If
        End Set
    End Property
#End Region

#Region "Not Mapped"
    Public ReadOnly Property short_comment As String
        Get
            If comment = "" Then
                Return ""
            Else
                'Trim after 40 signs and remove Line Feeds
                Dim v_temp As String = comment.Replace(Environment.NewLine, " ")
                If v_temp.Length < 40 Then
                    Return v_temp
                Else
                    Return v_temp.Substring(0, 39) & "..."
                End If
            End If
        End Get
    End Property

    Dim v_current_value As Decimal = 0
    <NotMapped>
    Public Property current_value As Decimal
        Get
            Return v_current_value
        End Get
        Set(value As Decimal)
            If v_current_value <> value Then
                v_current_value = value
                NotifyPropertyChanged("current_value")
                NotifyPropertyChanged("current_value_perc")
            End If
        End Set
    End Property


    Public ReadOnly Property current_value_perc As Decimal
        Get
            If current_value = 0 AndAlso target_value = 0 Then Return 0
            Return current_value / target_value
        End Get
    End Property

    Dim v_realstartdate As Date
    <NotMapped>
    Public Property RealStartDate As Date
        Get
            Return v_realstartdate
        End Get
        Set(value As Date)
            If v_realstartdate <> value Then
                v_realstartdate = value
                NotifyPropertyChanged("RealStartDate")
            End If
        End Set
    End Property

    Dim v_realenddate As Date
    <NotMapped>
    Public Property RealEndDate As Date
        Get
            Return v_realenddate
        End Get
        Set(value As Date)
            If v_realenddate <> value Then
                v_realenddate = value
                NotifyPropertyChanged("RealEndDate")
            End If
        End Set
    End Property

#End Region

End Class
