﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema

Public Class trans_typ_v01
    Inherits model_base

    'Interne Variablen
    Dim v_typ_id As Integer
    Dim v_internal_name As String
    Dim v_bezeichnung As String
    Dim v_color As String = "#000000"
    Dim v_shortcut_key As String
    Dim v_multiplier As Short

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "bezeichnung"
                    If String.IsNullOrEmpty(bezeichnung) Then result = "Bezeichnung darf nicht leer sein"
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property typ_id As Integer
        Get
            Return v_typ_id
        End Get
        Set(value As Integer)
            If v_typ_id <> value Then
                v_typ_id = value
                NotifyPropertyChanged("typ_id")
            End If
        End Set
    End Property

    Public Property internal_name As String
        Get
            Return v_internal_name
        End Get
        Set(value As String)
            If v_internal_name <> value Then
                v_internal_name = value
                NotifyPropertyChanged("internal_name")
            End If
        End Set
    End Property

    ''' <summary>
    ''' Multiplizierer:
    ''' Wenn internal_name ein = 1
    ''' aus = -1
    ''' transfer = 0
    ''' </summary>
    ''' <returns></returns>
    Public Property multiplier As Short
        Get
            Return v_multiplier
        End Get
        Set(value As Short)
            If v_multiplier <> value Then
                v_multiplier = value
                NotifyPropertyChanged("multiplier")
            End If
        End Set
    End Property


    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If v_bezeichnung <> value Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
            End If
        End Set
    End Property

    Public Property color As String
        Get
            Return v_color
        End Get
        Set(value As String)
            If v_color <> value Then
                v_color = value
                NotifyPropertyChanged("color")
                NotifyPropertyChanged("solid_color")
            End If
        End Set
    End Property

    Public Property shortcut_key As String
        Get
            Return v_shortcut_key
        End Get
        Set(value As String)
            If v_shortcut_key <> value Then
                v_shortcut_key = value
                NotifyPropertyChanged("shortcut_key")
            End If
        End Set
    End Property
#End Region

#Region "Navigation Properties"
    Public Overridable Property nv_transtyp_trans As New ObservableCollection(Of transactions_v01)
    Public Overridable Property trans_rep As New ObservableCollection(Of transactions_repeat_v01)
    Public Overridable Property nv_transtyp_2_transtemp As New ObservableCollection(Of transactions_template_v01)
#End Region


#Region "Extension"
    ''' <summary>
    ''' Property offers a SolidColorBrush out of the String HTML Color to change Foreground Color of an Item.
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property solid_color() As SolidColorBrush
        Get
            If color Is Nothing Then
                Return ColorBrushFromResources("IdealForegroundColorBrush")
            Else
                Try
                    Dim v_col As System.Drawing.Color = System.Drawing.ColorTranslator.FromHtml(color)
                    Dim v_color As Color = System.Windows.Media.Color.FromArgb(v_col.A, v_col.R, v_col.G, v_col.B)
                    Return New SolidColorBrush(v_color)
                Catch ex As Exception
                    Return ColorBrushFromResources("IdealForegroundColorBrush")
                End Try
            End If
        End Get
    End Property

    Public ReadOnly Property symbol_ever() As Byte()
        Get
            Select Case internal_name
                Case "ein"
                    Return ResourceDictionaryImagetoByte("typ_plus")
                Case "aus"
                    Return ResourceDictionaryImagetoByte("typ_minus")
                Case "transfer"
                    Return ResourceDictionaryImagetoByte("typ_transfer")
                Case Else
                    Return ResourceDictionaryImagetoByte("Error")
            End Select
        End Get
    End Property

    ''' <summary>
    ''' Returns a String Value of the Typ Typ...
    ''' used when copy data out of datagrid
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property typ_str As String
        Get
            Select Case internal_name
                Case "ein"
                    Return "+"
                Case "aus"
                    Return "-"
                Case "transfer"
                    Return "+/-"
                Case Else
                    Return ""
            End Select
        End Get
    End Property
#End Region


End Class
