﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Windows.Shell
Imports MahApps.Metro.Controls.Dialogs

Namespace HHHB3.ViewModel
    Partial Public Class MainViewModel

#Region "Special Folders"
        Private WithEvents __specialfolders As New ObservableCollection(Of cls_specialfolderitem)
        Public Property SpecialFolders As ListCollectionView


        Private Sub InitSpecialFolders()
            'Ordner hinzufügen:
            With __specialfolders
                .Add(New cls_specialfolderitem With {.FolderName = "Desktop", .FolderPath = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory), .FolderPicture = FindResourceImage("Desktop")})
                .Add(New cls_specialfolderitem With {.FolderName = "Dokumente", .FolderPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), .FolderPicture = FindResourceImage("DocumentFolder")})
                .Add(New cls_specialfolderitem With {.FolderName = "Dieser Computer", .FolderPath = Environment.GetFolderPath(Environment.SpecialFolder.MyComputer), .FolderPicture = FindResourceImage("Computer")})
                If SettingsCurrent.backup_onstartup Or SettingsCurrent.backup_onshutdown Then
                    .Add(New cls_specialfolderitem With {.FolderName = "Backupordner", .FolderPath = SettingsCurrent.backup_path, .FolderPicture = FindResourceImage("BackupFolder")})
                End If
                .Add(New cls_specialfolderitem With {.FolderName = "Durchsuchen", .FolderPath = "", .FolderPicture = FindResourceImage("SearchFile")})
            End With
            SpecialFolders = New ListCollectionView(__specialfolders)
        End Sub
#End Region

#Region "Last Used Databases"
        Private WithEvents __LastUsedDBs As New ObservableCollection(Of cls_lastdb)
        Public Property LastUsedDBs_pinned As ListCollectionView
        Public Property LastUsedDBs_unpinned As ListCollectionView
        Public Property LastUsedDBs As ListCollectionView

        Dim fil_lastuseddbs As New cls_filter_lastuseddb
        Private Sub InitLastUsedDBs()
            __LastUsedDBs.Clear()
            __LastUsedDBs = get_lastdbs_from_file()
            If __LastUsedDBs Is Nothing Then __LastUsedDBs = New ObservableCollection(Of cls_lastdb)
            LastUsedDBs_pinned = New ListCollectionView(__LastUsedDBs) With {
                .Filter = New Predicate(Of Object)(AddressOf fil_lastuseddbs.PinnedDBs)
            }
            LastUsedDBs_unpinned = New ListCollectionView(__LastUsedDBs) With {
                .Filter = New Predicate(Of Object)(AddressOf fil_lastuseddbs.UnPinnedDBs)
            }
            LastUsedDBs = New ListCollectionView(__LastUsedDBs)
            LastUsedDBs_pinned.MoveCurrentToFirst()
            LastUsedDBs_unpinned.MoveCurrentToFirst()
            LastUsedDBs.MoveCurrentToFirst()
        End Sub

        Private Sub __LastUsedDBs_CollectionChanged(sender As Object, e As NotifyCollectionChangedEventArgs) Handles __LastUsedDBs.CollectionChanged
            If e.Action <> NotifyCollectionChangedAction.Remove Then Return
            If Not LastUsedDBs.MoveCurrentToNext Then LastUsedDBs.MoveCurrentToPrevious()
            If Not LastUsedDBs_pinned.MoveCurrentToNext Then LastUsedDBs_pinned.MoveCurrentToPrevious()
            If Not LastUsedDBs_unpinned.MoveCurrentToNext Then LastUsedDBs_unpinned.MoveCurrentToPrevious()
            save_lastdb_to_file()
        End Sub
#End Region

#Region "About-Properties"
        Public Property AppTitle As String
        Public Property AppProductname As String
        Public Property AppVersion As String
        Public Property AppCopyright As String
        Public Property AppVendor As String
        Public Property AppDescription As String

        Private Sub AboutLoad()
            If My.Application.Info.Title <> "" Then
                AppTitle = My.Application.Info.Title
            Else
                AppTitle = System.IO.Path.GetFileNameWithoutExtension(My.Application.Info.AssemblyName)
            End If
            AppProductname = My.Application.Info.ProductName
            AppVersion = My.Application.Info.Version.ToString
            AppCopyright = My.Application.Info.Copyright
            AppVendor = My.Application.Info.CompanyName
            AppDescription = My.Application.Info.Description
        End Sub

#End Region


#Region "Database Management"
        Public Property cmdNewDatabase As New RelayCommand(Of String)(Sub(p) NewDatabase(p))

        ''' <summary>
        ''' Creates a new database
        ''' </summary>
        Private Sub NewDatabase(Optional v_filename As String = "")
            'Dim objectContext As Core.Objects.ObjectContext = CType(dbo, Infrastructure.IObjectContextAdapter).ObjectContext
            'objectContext.
            'https://social.msdn.microsoft.com/Forums/en-US/42063f4f-d714-43a9-8089-e824a7c206c4/how-do-i-create-a-new-database-file-at-runtime-with-a-predefined-schema-in-entity-framework?forum=adodotnetentityframework
            'dbo.Database.Create()
            'Neue Datenbank anlegen.
            'Datenbank speichern untern:
            Dim objdialog As New Forms.SaveFileDialog
            With objdialog
                If v_filename IsNot "" Then .FileName = v_filename & ".hhdb3"
                .Filter = "HHHB3 Datenbank (*.hhdb3)|*.hhdb3"
                .Title = "Datenbank erstellen"
            End With
            If objdialog.ShowDialog() = Forms.DialogResult.OK Then
                If objdialog.FileName <> "" Then
                    cls_logger.v_logger.Info("DBLastUsed before: " & GetValueFromRegistryKey("db_lastused"))
                    SetValueFromRegistryKey("db_lastused", objdialog.FileName)
                    cls_logger.v_logger.Info("DBLastUsed after: " & GetValueFromRegistryKey("db_lastused"))
                    If isRestart = False Then System.Diagnostics.Process.Start(Application.ResourceAssembly.Location)
                    Environment.Exit(0)
                End If
            End If
        End Sub

        Public Property cmdDatabaseSaveTo As New RelayCommand(Of String)(Sub(p) DatabaseSaveTo(p))

        Private Async Sub DatabaseSaveTo(Optional v_name As String = "")
            'Datenbank speichern untern:
            Dim objdialog As New Forms.SaveFileDialog
            With objdialog
                If v_name IsNot "" Then .FileName = v_name & ".hhdb3"
                .Filter = "HHHB3 Datenbank (*.hhdb3)|*.hhdb3"
                .Title = "Datenbank speichern"
            End With
            If objdialog.ShowDialog() = Forms.DialogResult.OK Then
                If objdialog.FileName <> "" Then
                    'Aktuelle Datenbank kopieren:
                    Dim v_lastused_db_path As String = GetValueFromRegistryKey("db_lastused")
                    Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)

                    If System.IO.File.Exists(v_lastused_db_path) Then
                        'Existiert also kopieren
                        IO.File.Copy(v_lastused_db_path, objdialog.FileName)
                        Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}
                        Dim result As MessageDialogResult = Await vMainWindow.ShowMessageAsync("Datenbank erfolgreich gespeichert", "Wollen sie zur neuen Datenbank wechseln?", MessageDialogStyle.AffirmativeAndNegative, dia_settings)
                        If result = MessageDialogResult.Affirmative Then
                            SetValueFromRegistryKey("db_lastused", objdialog.FileName)
                            If isRestart = False Then System.Diagnostics.Process.Start(Application.ResourceAssembly.Location)
                            Application.Current.Shutdown()
                            'Environment.Exit(0)
                        End If
                    Else
                        Await vMainWindow.ShowMessageAsync("Fehler", "Datenbank konnte nicht gespeichert werden", MessageDialogStyle.Affirmative)
                    End If
                End If
            End If

        End Sub


        Public Property cmdOpenDatabase As New RelayCommand(Of String)(Sub(p) OpenDatabase(p))
        Public Property cmdOpenDatabaseDirect As New RelayCommand(Of cls_lastdb)(Sub(p) OpenDatabaseDirect(p), Function(p) p IsNot Nothing)
        Public Property cmdAddDB2Fav As New RelayCommand(Of cls_lastdb)(Sub(p) AddDB2Fav(p), Function(p) p IsNot Nothing)
        Public Property cmdDeleteDBfromFav As New RelayCommand(Of cls_lastdb)(Sub(p) DeleteDB2Fav(p), Function(p) p IsNot Nothing)
        Public Property cmdDeleteDatabaseLU As New RelayCommand(Of cls_lastdb)(Sub(p) DeleteLastUsedDB(p), Function(p) p IsNot Nothing)

        Private Async Sub OpenDatabase(Optional v_startpath As String = "")
            'Opens an existing database

            Dim objDialog As Forms.OpenFileDialog = New Forms.OpenFileDialog
            With objDialog
                If v_startpath IsNot "" Then .InitialDirectory = v_startpath
                .DefaultExt = ".hhdb3"
                .Filter = "HHHB3 Datenbank (*.hhdb3)|*.hhdb3|Generische Datenbank (*.sdf)|*.sdf"
            End With

            Dim objResult As Forms.DialogResult = objDialog.ShowDialog
            If objResult = Forms.DialogResult.OK Then


                Dim v_oldpath As String = GetValueFromRegistryKey("db_lastused")
                SetValueFromRegistryKey("db_lastused", objDialog.FileName)
                'Prüfen ob die DB korrekt ist:
                If check_db(GetValueFromRegistryKey("db_lastused")) Then
                    'Datenbank austauschen
                    set_db()
                    dbo.Database.Connection.ConnectionString = set_db_constring()
                    System.Diagnostics.Process.Start(Application.ResourceAssembly.Location)
                    'Application.Current.Shutdown()
                    Environment.Exit(0)
                Else
                    Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)
                    Await vMainWindow.ShowMessageAsync("Fehler", "Datenbank konnte nicht gefunden werden", MessageDialogStyle.Affirmative)
                End If
            End If
        End Sub

        Private Async Sub OpenDatabaseDirect(v_db As cls_lastdb)
            If v_db Is Nothing Then Exit Sub
            Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)
            Dim result As MessageDialogResult = Await vMainWindow.ShowMessageAsync("Datenbank öffnen", "Wollen Sie auf die neue Datenbank wechseln?", MessageDialogStyle.AffirmativeAndNegative, New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"})
            If result = MessageDialogResult.Affirmative Then
                If check_db(v_db.pfad) Then
                    SetValueFromRegistryKey("db_lastused", v_db.pfad)
                    set_db()
                    dbo.Database.Connection.ConnectionString = set_db_constring()
                    If isRestart = False Then System.Diagnostics.Process.Start(Application.ResourceAssembly.Location)
                    Environment.Exit(0)
                    'Application.Current.Shutdown()
                Else
                    Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}
                    result = Await vMainWindow.ShowMessageAsync("Datenbank konnte nicht gefunden werden", "Eintrag aus der Liste entfernen?", MessageDialogStyle.AffirmativeAndNegative, dia_settings)
                    If result = MessageDialogResult.Affirmative Then DeleteLastUsedDB(v_db)
                End If
            End If
        End Sub

        Private Sub AddDB2Fav(v_db As cls_lastdb)
            If v_db Is Nothing Then Exit Sub
            With v_db
                .BeginEdit()
                .pinned = True
                .EndEdit()
            End With
            LastUsedDBs_pinned.Filter = New Predicate(Of Object)(AddressOf fil_lastuseddbs.PinnedDBs)
            LastUsedDBs_unpinned.Filter = New Predicate(Of Object)(AddressOf fil_lastuseddbs.UnPinnedDBs)
            LastUsedDBs_pinned.MoveCurrentToFirst()
            LastUsedDBs_unpinned.MoveCurrentToFirst()
            save_lastdb_to_file()
        End Sub

        Private Sub DeleteDB2Fav(v_db As cls_lastdb)
            If v_db Is Nothing Then Exit Sub
            With v_db
                .BeginEdit()
                .pinned = False
                .EndEdit()
            End With
            LastUsedDBs_pinned.Filter = New Predicate(Of Object)(AddressOf fil_lastuseddbs.PinnedDBs)
            LastUsedDBs_unpinned.Filter = New Predicate(Of Object)(AddressOf fil_lastuseddbs.UnPinnedDBs)
            save_lastdb_to_file()
        End Sub

        Private Sub DeleteLastUsedDB(v_db As cls_lastdb)
            If v_db Is Nothing Then Exit Sub
            __LastUsedDBs.Remove(v_db)
            save_lastdb_to_file()
        End Sub

        Private Async Sub save_lastdb_to_file()
            Dim _xmlhandler As New cls_xml_handler
            Dim v_filepath As String = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3\lastuseddb.xml"
            If System.IO.File.Exists(v_filepath) Then IO.File.Delete(v_filepath)
            If Not System.IO.File.Exists(v_filepath) Then
                If Not IO.Directory.Exists(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt") Then IO.Directory.CreateDirectory(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt")
                If Not IO.Directory.Exists(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3") Then IO.Directory.CreateDirectory(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3")
                If _xmlhandler.create_new_xml_config_file("lastuseddb.xml", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3\", __LastUsedDBs) = cls_xml_handler.e_status.nok Then
                    Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)
                    If Not vMainWindow Is Nothing Then Await vMainWindow.ShowMessageAsync("Fehler", "Zuletzt benutzte Datenbanken wurden nicht gesichert", MessageDialogStyle.Affirmative)
                    Exit Sub
                End If
            End If
        End Sub

        Private Function get_lastdbs_from_file() As ObservableCollection(Of cls_lastdb)
            'XML file is stored in %appdata%
            Dim v_filepath As String = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3\lastuseddb.xml"
            If Not System.IO.File.Exists(v_filepath) Then
                'Create for the next time:
                If Not IO.Directory.Exists(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt") Then IO.Directory.CreateDirectory(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt")
                If Not IO.Directory.Exists(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3") Then IO.Directory.CreateDirectory(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3")
                If Not IO.File.Exists(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3\lastuseddb.xml") Then
                    'Write the XML Stuff to the document:
                    Dim v_dblastused_doc As XDocument = <?xml version="1.0" encoding="UTF-8"?>
                                                        <Databases>
                                                        </Databases>
                    v_dblastused_doc.Save(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) & "\MyAreaAt\HHHB3\lastuseddb.xml")
                    v_dblastused_doc = Nothing
                End If
                Return Nothing
            End If
            'File exists now read databases:
            'Temporary Collection:
            Dim v_temp_lvwlist As New ObservableCollection(Of cls_lastdb)
            Dim v_xmlfile As New Xml.XmlDocument

            Try
                With v_xmlfile
                    .Load(v_filepath)
                    'Does the root element Databases exist?
                    If Not .DocumentElement.Name = "Databases" Then Return Nothing
                    'are there any childs?
                    If Not .DocumentElement.HasChildNodes Then Return Nothing

                    For i As Integer = 0 To .DocumentElement.ChildNodes.Count - 1
                        Debug.Print(.DocumentElement.ChildNodes(i).Name())
                        If .DocumentElement.ChildNodes(i).Name = "Database" Then
                            'Found a database, check children:
                            Dim v_temp_name As String = ""
                            Dim v_temp_path As String = ""
                            Dim v_temp_pinned As Boolean = False
                            Dim v_temp_sorter As Integer = 0
                            Dim v_temp_date As Date = Date.Now
                            For z As Integer = 0 To .DocumentElement.ChildNodes(i).Attributes.Count - 1
                                Select Case .DocumentElement.ChildNodes(i).Attributes(z).Name
                                    Case "DBName"
                                        v_temp_name = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                    Case "Path"
                                        v_temp_path = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                    Case "Pinned"
                                        Dim v_temp_pinned_string As String = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                        If v_temp_pinned_string = "True" Then
                                            v_temp_pinned = True
                                        Else
                                            v_temp_pinned = False
                                        End If
                                    Case "Sorter"
                                        Dim v_temp_order_string As String = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                        v_temp_sorter = CInt(v_temp_order_string)
                                    Case "Add_Date"
                                        Dim v_temp_date_string As String = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                        v_temp_date = ConvertToDateTime(v_temp_date_string)
                                        If v_temp_date = Nothing Then v_temp_date = Date.Now
                                End Select
                            Next
                            v_temp_lvwlist.Add(New cls_lastdb() With {.dbname = v_temp_name, .pfad = v_temp_path, .pinned = v_temp_pinned, .sorter = v_temp_sorter, .add_date = v_temp_date})
                            v_temp_name = ""
                            v_temp_path = ""
                            v_temp_pinned = False
                            v_temp_date = Date.Now

                        End If
                    Next
                End With

            Catch ex As Exception
                Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)
                If vMainWindow IsNot Nothing Then vMainWindow.ShowModalMessageExternal("Fehler", ex.Message, MessageDialogStyle.Affirmative)
                cls_logger.v_logger.Error("Fehler bei get_last_db: " & ex.Message)
                Return Nothing
            End Try
            Return v_temp_lvwlist
        End Function

        Private Sub activate_jumplist()

            Dim v_curjumplist As JumpList = JumpList.GetJumpList(Application.Current)
            If v_curjumplist Is Nothing Then Exit Sub

            For Each item As cls_lastdb In __LastUsedDBs
                v_curjumplist.JumpItems.Add(item.jump_path)
            Next
            v_curjumplist.ShowFrequentCategory = False
            v_curjumplist.Apply()

            cls_logger.v_logger.Info("Jumplist wurde gesetzt")
            cls_logger.v_logger.Info("Jumplist applied")
        End Sub


#End Region

    End Class
End Namespace

