﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Data.Entity
Imports System.IO
Imports System.Text
Imports MahApps.Metro.Controls.Dialogs

Namespace HHHB3.ViewModel
    Partial Public Class MainViewModel
        'Partial Entität Documents:

        Enum Dokmanager_action
            add
            edit
            delete
            open
        End Enum

        Private WithEvents __documents As New ObservableCollection(Of documents_v01)()
        Public Property DocumentList() As ListCollectionView

        Private Sub DocumentsLoad()
            __documents.Clear()
            dbo.m_documents_v01.Load
            __documents = dbo.m_documents_v01.Local
            DocumentList = New ListCollectionView(__documents)
            DocumentList.MoveCurrentToFirst()
            'necessary to avoid an bug which causes the duplicate and delete command to be switched of if just 1 item is available:
            DocumentsSelectedItems = __documents.ToList
            DocumentsSelectedItems.Clear()
            DocumentsSelectedItems.Add(__documents.FirstOrDefault)

        End Sub

        Private Sub _documents_CollectionChanged(sender As Object, e As Specialized.NotifyCollectionChangedEventArgs) Handles __documents.CollectionChanged
            If Not DocumentList Is Nothing Then
                If Not DocumentList.MoveCurrentToNext Then DocumentList.MoveCurrentToPrevious()
            End If
        End Sub

        'commands
        Public Property cmdDocumentFilterView As New RelayCommand(Of cls_timeframe)(Sub(p) DocumentsFilterView(p))
        ''' <summary>
        ''' This command is necessary to receive the currently selected Repeat Transactions from the containing listview of the view
        ''' Command is necessary because datagrid selecteditems is not a dependency property see:
        ''' http://blog.galasoft.ch/posts/2010/05/handling-datagrid-selecteditems-in-an-mvvm-friendly-manner/
        ''' </summary>
        ''' <returns></returns>
        Public Property cmdDocumentsSelectionChanged As New RelayCommand(Of IList)(Sub(p) DocumentsSelectionChanged(p))
        Public Property cmdDocumentsShowTransactions As New RelayCommand(AddressOf DocumentsShowTransactions, Function() DocumentsShowCanExecute())
        Public Property cmdDocumentsShowFile As New RelayCommand(AddressOf DocumentsShowFile, Function() DocumentsShowCanExecute())
        Public Property cmdDocumentsShowTrash As New RelayCommand(AddressOf DocumentsShowTrash, Function() DocumentsShowDeleteTrashCanExecute())
        Public Property cmdDocumentsDeleteTrash As New RelayCommand(AddressOf DocumentsDeleteTrash, Function() DocumentsShowDeleteTrashCanExecute())

        ''' <summary>
        ''' Filters the shown timeframe of documents
        ''' </summary>
        ''' <param name="v_zeitraum">Zeitraum gewählt</param>
        Private Sub DocumentsFilterView(v_zeitraum As cls_timeframe)
            If v_zeitraum Is Nothing Then Exit Sub
            'Check if a filter is set and if so then remove it.
            If DocumentList.IsLiveFiltering Then DocumentList.Filter = Nothing
            If DocumentList.Filter Is Nothing Then DocumentList.Filter = Nothing

            Dim v_startdate As Date
            Dim v_enddate As Date

            Select Case v_zeitraum.Id
                Case 2
                    'Current Month
                    v_startdate = New Date(Date.Now.Year, Date.Now.Month, 1)
                    v_enddate = v_startdate.AddMonths(1).AddDays(-1)
                    'Set the Dates to pass them to the GUI
                    SetDocumentShowDates(v_startdate.ToShortDateString, v_enddate.ToShortDateString)
                Case 3
                    'Month before
                    'Änderung: 1.0.1.3 Berechnung des Start Monats geändert wg. Problem mit Jahreswechsel
                    v_startdate = New Date(Date.Now.Year, Date.Now.Month, 1)
                    v_startdate = v_startdate.AddMonths(-1)
                    v_enddate = v_startdate.AddMonths(1).AddDays(-1)
                    'Set the Dates to pass them to the GUI
                    SetDocumentShowDates(v_startdate.ToShortDateString, v_enddate.ToShortDateString)
                Case 4
                    'Last 2 Months
                    'Änderung: 1.0.1.3 Berechnung des Start Monats geändert wg. Problem mit Jahreswechsel
                    v_startdate = New Date(Date.Now.Year, Date.Now.Month, 1)
                    v_startdate = v_startdate.AddMonths(-2)
                    v_enddate = v_startdate.AddMonths(2).AddDays(-1)
                    'Set the Dates to pass them to the GUI
                    SetDocumentShowDates(v_startdate.ToShortDateString, v_enddate.ToShortDateString)
                Case 5
                    'Current quarter
                    Select Case Date.Now.Month / 3
                        Case <= 1
                            '1.Quartal
                            v_startdate = New Date(Date.Now.Year, 1, 1)
                        Case <= 2
                            '2.Quartal
                            v_startdate = New Date(Date.Now.Year, 4, 1)
                        Case <= 3
                            '3.Quartal
                            v_startdate = New Date(Date.Now.Year, 7, 1)
                        Case <= 4
                            '4.Quartal
                            v_startdate = New Date(Date.Now.Year, 10, 1)
                    End Select
                    v_enddate = v_startdate.AddMonths(3).AddDays(-1)
                    'Set the Dates to pass them to the GUI
                    SetDocumentShowDates(v_startdate.ToShortDateString, v_enddate.ToShortDateString)
                Case 6
                    'Current halfyear
                    If Date.Now.Month <= 6 Then
                        '1.halfyear
                        v_startdate = New Date(Date.Now.Year, 1, 1)
                        v_enddate = v_startdate.AddMonths(6)
                    Else
                        '2.halfyear
                        v_startdate = New Date(Date.Now.Year, 7, 1)
                        v_enddate = New Date(Date.Now.Year, 1, 1).AddYears(1)
                    End If
                    'Set the Dates to pass them to the GUI
                    Dim v_enddate_real = v_enddate.AddDays(-1)
                    SetDocumentShowDates(v_startdate.ToShortDateString, v_enddate_real.ToShortDateString)
                Case 7
                    'Current Year
                    v_startdate = New Date(Date.Now.Year, 1, 1)
                    v_enddate = New Date(Date.Now.Year, 1, 1).AddYears(1)
                    'Set the Dates to pass them to the GUI
                    Dim v_enddate_real = v_enddate.AddDays(-1)
                    SetDocumentShowDates(v_startdate.ToShortDateString, v_enddate_real.ToShortDateString)
                Case 8
                    'Free Timeframe
                    Dim v_selector As New frm_timeframe
                    Dim v_timeframe As New cls_timeframe_defined With {.start_date = New Date(Date.Now.Year, Date.Now.Month, 1), .end_date = New Date(Date.Now.Year, Date.Now.Month, DateTime.DaysInMonth(Date.Now.Year, Date.Now.Month))}
                    With v_selector
                        .DataContext = v_timeframe
                        .ShowDialog()
                        If .DialogResult Then
                            v_startdate = v_timeframe.start_date
                            v_enddate = v_timeframe.end_date
                            'Set the Dates to pass them to the GUI
                            SetDocumentShowDates(v_startdate.ToShortDateString, v_enddate.ToShortDateString)
                            'check if last day in month
                            If v_enddate.Day = Date.DaysInMonth(v_enddate.Year, v_enddate.Month) Then
                                v_enddate = New Date(v_enddate.Year, v_enddate.Month, 1).AddMonths(1)
                            Else
                                'move tomeframe 1 day before
                                v_enddate = v_enddate.AddDays(1)
                            End If
                        Else
                            Exit Sub
                        End If
                    End With

            End Select
            'Änderung 01.05.2017 Filter erst nach der Einstellung der Datumswerte entfernen optische Verbesserung:
            'Prüfen ob nicht bereits ein Filter aktiv ist:
            If DocumentList.IsLiveFiltering Then DocumentList.Filter = Nothing
            If Not DocumentList.Filter Is Nothing Then DocumentList.Filter = Nothing

            'Bevor filtering gleich prüfen ob nicht alle Buchungen gewählt ist:
            If v_zeitraum.Id = 1 Then
                DocumentList.Filter = Nothing
                SetDocumentShowDates("", "")
                Exit Sub
            End If

            DocumentList.Filter = Function(o) TryCast(o, documents_v01).nv_trans_2_doc_back.datum >= v_startdate And TryCast(o, documents_v01).nv_trans_2_doc_back.datum <= v_enddate
            DocumentList.Refresh()
        End Sub

        'Display the from and to dates:
        Dim m_docshowfrom As String = "-"
        Dim m_docshowto As String = "-"

        Public Sub SetDocumentShowDates(v_start As String, v_end As String)
            If v_start = "" Then v_start = "-"
            If v_end = "" Then v_end = "-"
            DocShowFrom = v_start
            DocShowTo = v_end
        End Sub

        Public Property DocShowFrom As String
            Get
                Return m_docshowfrom
            End Get
            Set(value As String)
                If m_docshowfrom <> value Then
                    m_docshowfrom = value
                    RaisePropertyChanged("DocShowFrom")
                End If
            End Set
        End Property

        Public Property DocShowTo As String
            Get
                Return m_docshowto
            End Get
            Set(value As String)
                If m_docshowto <> value Then
                    m_docshowto = value
                    RaisePropertyChanged("DocShowTo")
                End If
            End Set
        End Property

        Dim DocumentsSelectedItems As IList(Of documents_v01) = Nothing

        Private Sub DocumentsSelectionChanged(v_selected_items As IList)
            If Not DocumentsSelectedItems Is Nothing Then DocumentsSelectedItems.Clear()
            DocumentsSelectedItems = CType(v_selected_items, System.Collections.IList).Cast(Of documents_v01).Cast(Of documents_v01).ToList
        End Sub

        Private Async Sub DocumentsShowTransactions()
            If DocumentsSelectedItems Is Nothing Or DocumentsSelectedItems.Count = 0 Then Exit Sub
            'Shows the related transactions to a document in edit mode.
            If DocumentsSelectedItems.Count > 1 Then
                'if more than 1 Document is selected ask the user if he wants to proceed:
                Dim v_strbuilder As New StringBuilder
                v_strbuilder.AppendLine("Hinweis: Es werden der Reihe nach {0} Buchungen angezeigt.")
                v_strbuilder.AppendLine("Wollen Sie fortfahren?")
                Dim v_msg As String = String.Format(v_strbuilder.ToString, DocumentsSelectedItems.Count.ToString)
                Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}
                Dim v_mainwindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow)
                Dim result As MessageDialogResult = Await v_mainwindow.ShowMessageAsync("Buchungen anzeigen", v_msg, style:=MessageDialogStyle.AffirmativeAndNegative, settings:=dia_settings)
                If result = MessageDialogResult.Affirmative Then
                    For Each v_item As documents_v01 In DocumentsSelectedItems
                        If Not v_item Is Nothing Then TransactionEdit(v_item.nv_trans_2_doc_back)
                    Next
                End If
            Else
                TransactionEdit(DocumentsSelectedItems.First.nv_trans_2_doc_back)
            End If
        End Sub

        Private Sub DocumentsShowFile()
            If DocumentsSelectedItems Is Nothing Or DocumentsSelectedItems.Count = 0 Then Exit Sub
            'Shows the related files in the suitable programm.
            For Each v_item As documents_v01 In DocumentsSelectedItems
                If Not v_item Is Nothing Then DocRepositoryManager(v_item.path, Dokmanager_action.open)
            Next
        End Sub

        Private Function DocumentsShowCanExecute() As Boolean
            If DocumentsSelectedItems Is Nothing OrElse DocumentsSelectedItems.Count = 0 Then Return False
            If DocumentsSelectedItems.Count = 1 Then
                If DocumentsSelectedItems.First Is Nothing Then Return False
            End If
            Return True
        End Function

        Private Sub DocumentsShowTrash()
            'Shows Trash Folder in Windows Explorer:
            Dim v_docpath As String = SettingsCurrent.doc_path
            If Not v_docpath.EndsWith("\") Then v_docpath &= "\"

            If Directory.Exists(v_docpath & "wastebin") Then
                Process.Start(v_docpath & "wastebin")
            End If
        End Sub

        Private Async Sub DocumentsDeleteTrash()
            'Deletes the Trash Folder in Windows Explorer:
            Dim v_docpath As String = SettingsCurrent.doc_path
            If Not v_docpath.EndsWith("\") Then v_docpath &= "\"
            If Directory.Exists(v_docpath & "wastebin") Then
                Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}
                Dim v_mainwindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow)
                Dim result As MessageDialogResult = Await v_mainwindow.ShowMessageAsync("Papierkorb leeren", "Wollen Sie den Dokumentenpapierkorb von HHHB3 löschen?", MessageDialogStyle.AffirmativeAndNegative, dia_settings)
                If result = MessageDialogResult.Affirmative Then
                    Directory.Delete(v_docpath & "wastebin", True)
                    Directory.CreateDirectory(v_docpath & "wastebin")
                End If
            End If

        End Sub

        Private Function DocumentsShowDeleteTrashCanExecute() As Boolean
            'CBool(SettingsCurrent.doc_move_if_deleted) AndAlso SettingsCurrent.doc_path IsNot String.Empty
            If SettingsCurrent.doc_move_if_deleted AndAlso SettingsCurrent.doc_path IsNot String.Empty AndAlso System.IO.Directory.Exists(SettingsCurrent.doc_path) Then Return True
            Return False
        End Function


        ''' <summary>
        ''' The document Repository Managers performes CRUD Operations for a specified Dokument, deals directly with FileSystem
        ''' </summary>
        ''' <param name="org_path">Original Local Path of the File</param>
        ''' <param name="action">Add,Edit,Delete</param>
        ''' <returns>Returns Filename or relative Path for Saving in Database depends if subfolders are used or not</returns>
        Private Function DocRepositoryManager(org_path As String, action As Dokmanager_action) As String
            Dim vCurrentWindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.Windows.OfType(Of Window).SingleOrDefault(Function(x) x.IsActive), MahApps.Metro.Controls.MetroWindow)
            Dim v_newpath As String = ""
                Dim v_docpath As String = SettingsCurrent.doc_path
                If Not v_docpath.EndsWith("\") Then v_docpath &= "\"
            'Sicherheitsfunktion prüfen ob das Repository noch verfügbar ist:
            If Not My.Computer.FileSystem.DirectoryExists(v_docpath) Then
                vCurrentWindow.ShowMessageAsync("Fehler", "Dokument konnte nicht hinzugefügt werden, die Dokumentenablage ist nicht verfügbar", MessageDialogStyle.Affirmative)
                Return Nothing
            End If

            Select Case action
                    Case Dokmanager_action.add 'Hinzufügen
                        'If adding a file check if subfolder shall be used
                        If SettingsCurrent.doc_use_subfolder Then
                            'Subfolders shall be used
                            If Not My.Computer.FileSystem.DirectoryExists(v_docpath & Date.Now.ToString("yyyyMMdd")) Then
                                My.Computer.FileSystem.CreateDirectory(v_docpath & Date.Now.ToString("yyyyMMdd"))

                            End If
                            v_newpath = v_docpath & Date.Now.ToString("yyyyMMdd")
                        Else
                            'No Subfolder Unterordner:
                            v_newpath = v_docpath
                        End If
                    'Now copy the file to the repository:
                    Try
                        My.Computer.FileSystem.CopyFile(org_path, v_newpath & "\" & My.Computer.FileSystem.GetFileInfo(org_path).Name, True)
                    Catch ex As Exception
                        vCurrentWindow.ShowMessageAsync("Fehler", "Datei konnte nicht in die Dokumentenablage kopiert werden", MessageDialogStyle.Affirmative)
                        Return Nothing
                        End Try

                        'Wenn die Option aktiviert ist dann die Basisdatei löschen:

                        If SettingsCurrent.doc_delete_after_import Then
                            'Delete original File after import
                            My.Computer.FileSystem.DeleteFile(org_path)
                        End If

                        If SettingsCurrent.doc_use_subfolder Then
                            Return Date.Now.ToString("yyyyMMdd") & "\" & My.Computer.FileSystem.GetFileInfo(org_path).Name
                        Else
                            Return My.Computer.FileSystem.GetFileInfo(org_path).Name
                        End If
                    Case Dokmanager_action.delete  'Delete a file
                        If org_path = "" Then Return Nothing
                        'The Path is relativ so create the absolute path
                        Dim v_real_path As String = v_docpath & org_path
                        If My.Computer.FileSystem.FileExists(v_real_path) Then
                            'Es gibt 2 Fälle entweder Datei löschen oder in den Papierkorb verschieben:
                            'check if file should be moved to waste bin (internally HHHB not Windows!) or deleted
                            If SettingsCurrent.doc_move_if_deleted Then
                                'Use Waste bin
                                If Not My.Computer.FileSystem.DirectoryExists(v_docpath & "wastebin") Then
                                    My.Computer.FileSystem.CreateDirectory(v_docpath & "wastebin")
                                End If
                                My.Computer.FileSystem.MoveFile(v_real_path, v_docpath & "wastebin\" & My.Computer.FileSystem.GetFileInfo(v_real_path).Name, True)
                            Else
                            'Delete
                            Try
                                My.Computer.FileSystem.DeleteFile(v_real_path)
                            Catch ex As Exception
                                vCurrentWindow.ShowMessageAsync("Fehler", "Datei konnte nicht gelöscht werden", MessageDialogStyle.Affirmative)
                            End Try
                            End If
                        End If
                    Case Dokmanager_action.edit 'Editieren
                        Throw New NotImplementedException
                    Case Dokmanager_action.open 'Show File
                        If org_path = "" Then Return Nothing
                        Dim v_real_path As String = v_docpath & org_path
                    If My.Computer.FileSystem.FileExists(v_real_path) Then
                        'Open File
                        Process.Start(v_real_path)
                    Else
                        vCurrentWindow.ShowMessageAsync("Fehler", "Das anzuzeigende Dokument konnte in der Dokumentendatenbank nicht gefunden werden", MessageDialogStyle.Affirmative)
                    End If
                End Select
                Return Nothing
        End Function


    End Class
End Namespace