﻿Imports System.Collections.ObjectModel
Imports System.Data.Entity
Imports System.Text
Imports MahApps.Metro.Controls.Dialogs
Imports Microsoft.WindowsAPICodePack.Dialogs

Namespace HHHB3.ViewModel
    Partial Public Class MainViewModel
        'Partial: Settings
        Private WithEvents __settings As New ObservableCollection(Of settings)
        Private Property Settings As ListCollectionView
        ''' <summary>
        ''' Represents the current Settings Set.
        ''' </summary>
        ''' <returns></returns>
        Public Property SettingsCurrent As settings
            Get
                Return v_settingscurrent
            End Get
            Set(value As settings)
                v_settingscurrent = value
                RaisePropertyChanged("SettingsCurrent")
            End Set
        End Property
        Dim v_settingscurrent As settings


        Private Sub SettingsLoad()
            __settings.Clear()
            dbo.m_settings.Load
            __settings = dbo.m_settings.Local
            Settings = New ListCollectionView(__settings)
            If Settings.Count = 0 Then
                'there is no settingset create a new one for this machine:
                create_new_settingset()
            Else
                cls_logger.v_logger.Info("Maschinename: " & Environment.MachineName.ToString)
                'Es gibt mehrere Settings, das dem Computer zugeordnete  finden:
                Dim v_computer_settings As settings = __settings.Where(Function(p) p.computername = Environment.MachineName).FirstOrDefault
                cls_logger.v_logger.Info("Settinswhere ausgeführt")
                If v_computer_settings Is Nothing Then
                    'Nichts gefunden:
                    cls_logger.v_logger.Info("Settingset nicht gefunden")
                    create_new_settingset()
                Else
                    Settings.MoveCurrentTo(v_computer_settings)
                    SettingsCurrent = DirectCast(Settings.CurrentItem, settings)
                    cls_logger.v_logger.Info("Settingsset gefunden")
                End If
            End If
        End Sub

        Private Sub create_new_settingset()
            Try
                Settings.AddNewItem(New settings)
                Settings.CommitNew()
                Settings.MoveCurrentToLast()
                dbo.SaveChanges()
                SettingsCurrent = DirectCast(Settings.CurrentItem, settings)
                cls_logger.v_logger.Info("Settingset erfolgreich erzeugt")
            Catch ex As Exception
                cls_logger.v_logger.Error("Es konnte kein Settingset erzeugt werden")
                cls_logger.v_logger.Error(ex.Message.ToString)
                Throw New Exception("Einstellungen konnten nicht festgelegt werden." & Environment.NewLine & "Das Programm wird beendet" & Environment.NewLine & ex.Message.ToString)
            End Try
        End Sub

        Private Sub __settings_CollectionChanged(sender As Object, e As Specialized.NotifyCollectionChangedEventArgs) Handles __settings.CollectionChanged
            If e.Action <> Specialized.NotifyCollectionChangedAction.Remove Then Return
            If Not Settings.MoveCurrentToNext Then Settings.MoveCurrentToPrevious()
            UpdateReportsAvailable()
            'Update Journal import columns:
            Call SetBJImportColumns()
        End Sub

#Region "Commands"

        Enum DocTypes
            Doc
            DocWatch
            Backup
        End Enum


        Public Property cmdSaveSettings As New RelayCommand(Of Boolean)(Sub(p) SaveSettings(p), Function() SettingsCurrent IsNot Nothing)
        Public Property cmdCancelSettings As New RelayCommand(AddressOf CancelSettings, Function() SettingsCurrent IsNot Nothing OrElse SettingsCurrent.isinEdit = True)
        Public Property cmdGetDocumentPath() As New RelayCommand(Sub() GetDocPath(DocTypes.Doc))
        Public Property cmdGetDocumentWatchPath As New RelayCommand(Sub() GetDocPath(DocTypes.DocWatch))
        Public Property cmdGetBackupPath() As New RelayCommand(Sub() GetDocPath(DocTypes.Backup))
#End Region

        Private Async Sub SaveSettings(Optional close_always As Boolean = False)
            'Aktuelles Settingset speichern:
            If SettingsCurrent.isinEdit Then
                Dim v_strbuilder As New StringBuilder
                v_strbuilder.AppendLine("Änderungen speichern?")
                Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}
                Dim v_mainwindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow)
                Dim result As MessageDialogResult = Await v_mainwindow.ShowMessageAsync("Frage", v_strbuilder.ToString, MessageDialogStyle.AffirmativeAndNegative, dia_settings)

                If result = MessageDialogResult.Affirmative Then
                    If SettingsCurrent.doc_watcher_use Then
                        'check if it's necessary to reinit the docwathcer:
                        If SettingsCurrent.old_docwatcherpath <> SettingsCurrent.doc_watcher_path Then
                            InitDocWatcher()
                        End If
                    Else
                        doc_watcher.EnableRaisingEvents = False
                    End If
                    SettingsCurrent.EndEdit()
                    dbo.SaveChanges()

                    'Just in Case some "Actives" have changed reinit the Search Entities
                    SearchesLoad()
                    SettingsCurrent.Close()
                    'Now check the state of the dashboard lock:
                    If SettingsCurrent.home_locked Then
                        If Widgets.Contains(WidgetDummy) Then Widgets.Remove(WidgetDummy)
                    Else
                        If Not Widgets.Contains(WidgetDummy) Then Widgets.Add(WidgetDummy)
                    End If

                Else
                    If close_always Then
                        SettingsCurrent.CancelEdit()
                        SettingsCurrent.Close()
                    End If
                End If
            End If
            'Check for WidgetsAvailable:
            check_available_widgets()
        End Sub

        Private Async Sub CancelSettings()
            If SettingsCurrent.isinEdit Then
                Dim v_strbuilder As New StringBuilder
                v_strbuilder.AppendLine("Änderungen verwerfen?")
                Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}
                Dim v_mainwindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow)
                Dim result As MessageDialogResult = Await v_mainwindow.ShowMessageAsync("Frage", v_strbuilder.ToString, MessageDialogStyle.AffirmativeAndNegative, dia_settings)
                If result = MessageDialogResult.Affirmative Then
                    SettingsCurrent.CancelEdit()
                    SettingsCurrent.Close()
                End If
            End If
            'Check for WidgetsAvailable:
            check_available_widgets()
        End Sub

        Private Sub GetDocPath(v_doctype As DocTypes)
            Dim v_oldpath As String = String.Empty
            Dim v_newpath As String = String.Empty

            Select Case True
                Case v_doctype = DocTypes.Doc
                    v_oldpath = SettingsCurrent.doc_path
                Case v_doctype = DocTypes.Backup
                    v_oldpath = SettingsCurrent.backup_path
                Case v_doctype = DocTypes.DocWatch
                    v_oldpath = SettingsCurrent.doc_watcher_path
            End Select

            v_newpath = GetFolder(v_oldpath)
            If v_newpath = Nothing Then
                Exit Sub
            Else
                Select Case True
                    Case v_doctype = DocTypes.Doc
                        SettingsCurrent.doc_path = v_newpath
                    Case v_doctype = DocTypes.Backup
                        SettingsCurrent.backup_path = v_newpath
                    Case v_doctype = DocTypes.DocWatch
                        SettingsCurrent.doc_watcher_path = v_newpath
                End Select
            End If
        End Sub


        ''' <summary>
        ''' Opens CommonFileDialog to select a Folder path (not File!)
        ''' </summary>
        ''' <param name="v_oldpath"></param>
        ''' <returns>Selected FolderPath or nothing if no path is selected</returns>
        Private Function GetFolder(v_oldpath As String) As String
            Dim v_dia As New CommonOpenFileDialog With {.IsFolderPicker = True}
            If IO.Directory.Exists(v_oldpath) Then v_dia.DefaultDirectory = v_oldpath
            Dim v_result As CommonFileDialogResult = v_dia.ShowDialog
            If v_result = CommonFileDialogResult.Ok Then

                Return v_dia.FileName
            Else
                Return Nothing
            End If
        End Function

    End Class
End Namespace

